/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 *
 * Contributor: J. Polakovic
 */

package org.objectweb.fractal.cecilia.adl.types;

/**
 * Extends the default
 * {@link org.objectweb.fractal.adl.types.TypeInterface TypeInterface} AST node
 * to add the <code>startProperty</code> attribute.
 */
public interface TypeInterface
    extends
      org.objectweb.fractal.adl.types.TypeInterface {

  /**
   * The <code>startProperty</code> value for a client interface that must be
   * bound after the component is started.
   */
  String BINDAFTERSTART_STARTPROPERTY = "bindafterstart";
  
  /**
   * The <code>startProperty</code> value for a client interface that is used 
   * during the start (or the stop) of the component.
   */
  String USED_AT_START_START_PROPERTY = "usedAtStart";

  /**
   * Returns the value of the <code>startProperty</code> attribute.
   * 
   * @return the value of the <code>startProperty</code> attribute.
   */
  String getStartProperty();

  /**
   * Sets the value of the <code>startProperty</code> attribute.
   * 
   * @param property the value of the <code>startProperty</code> attribute.
   */
  void setStartProperty(String property);
}
